@@ -22,6 +22,7 @@
 
 import asyncio
 import itertools
+import logging
 import random
 import socket
 import sys
@@ -41,7 +42,8 @@ from ._handlers import (
 )
 from ._history import QuestionHistory
 from ._logger import QuietLogger, log
-from ._protocol import DNSIncoming, DNSOutgoing
+from ._protocol.incoming import DNSIncoming
+from ._protocol.outgoing import DNSOutgoing
 from ._services import ServiceListener
 from ._services.browser import ServiceBrowser
 from ._services.info import ServiceInfo, instance_name_from_service_info
@@ -210,16 +212,16 @@ class AsyncListener(asyncio.Protocol, QuietLogger):
     It requires registration with an Engine object in order to have
     the read() method called when a socket is available for reading."""
 
+    __slots__ = ('zc', 'data', 'last_time', 'transport', 'sock_description', '_deferred', '_timers')
+
     def __init__(self, zc: 'Zeroconf') -> None:
         self.zc = zc
         self.data: Optional[bytes] = None
         self.last_time: float = 0
         self.transport: Optional[asyncio.DatagramTransport] = None
-        self.sock_name: Optional[str] = None
-        self.sock_fileno: Optional[int] = None
+        self.sock_description: Optional[str] = None
         self._deferred: Dict[str, List[DNSIncoming]] = {}
         self._timers: Dict[str, asyncio.TimerHandle] = {}
-
         super().__init__()
 
     def suppress_duplicate_packet(self, data: bytes, now: float) -> bool:
@@ -235,6 +237,8 @@ class AsyncListener(asyncio.Protocol, QuietLogger):
     ) -> None:
         assert self.transport is not None
         v6_flow_scope: Union[Tuple[()], Tuple[int, int]] = ()
+        data_len = len(data)
+
         if len(addrs) == 2:
             # https://github.com/python/mypy/issues/1178
             addr, port = addrs  # type: ignore
@@ -252,32 +256,32 @@ class AsyncListener(asyncio.Protocol, QuietLogger):
                 'Ignoring duplicate message received from %r:%r [socket %s] (%d bytes) as [%r]',
                 addr,
                 port,
-                self._socket_description,
-                len(data),
+                self.sock_description,
+                data_len,
                 data,
             )
             return
 
-        if len(data) > _MAX_MSG_ABSOLUTE:
+        if data_len > _MAX_MSG_ABSOLUTE:
             # Guard against oversized packets to ensure bad implementations cannot overwhelm
             # the system.
             log.debug(
                 "Discarding incoming packet with length %s, which is larger "
                 "than the absolute maximum size of %s",
-                len(data),
+                data_len,
                 _MAX_MSG_ABSOLUTE,
             )
             return
 
-        msg = DNSIncoming(data, scope, now)
+        msg = DNSIncoming(data, (addr, port), scope, now)
         if msg.valid:
             log.debug(
                 'Received from %r:%r [socket %s]: %r (%d bytes) as [%r]',
                 addr,
                 port,
-                self._socket_description,
+                self.sock_description,
                 msg,
-                len(data),
+                data_len,
                 data,
             )
         else:
@@ -285,8 +289,8 @@ class AsyncListener(asyncio.Protocol, QuietLogger):
                 'Received from %r:%r [socket %s]: (%d bytes) [%r]',
                 addr,
                 port,
-                self._socket_description,
-                len(data),
+                self.sock_description,
+                data_len,
                 data,
             )
             return
@@ -345,29 +349,63 @@ class AsyncListener(asyncio.Protocol, QuietLogger):
 
         self.zc.handle_assembled_query(packets, addr, port, transport, v6_flow_scope)
 
-    @property
-    def _socket_description(self) -> str:
-        """A human readable description of the socket."""
-        return f"{self.sock_fileno} ({self.sock_name})"
-
     def error_received(self, exc: Exception) -> None:
         """Likely socket closed or IPv6."""
         # We preformat the message string with the socket as we want
         # log_exception_once to log a warrning message once PER EACH
         # different socket in case there are problems with multiple
         # sockets
-        msg_str = f"Error with socket {self._socket_description}): %s"
+        msg_str = f"Error with socket {self.sock_description}): %s"
         self.log_exception_once(exc, msg_str, exc)
 
     def connection_made(self, transport: asyncio.BaseTransport) -> None:
         self.transport = cast(asyncio.DatagramTransport, transport)
-        self.sock_name = self.transport.get_extra_info('sockname')
-        self.sock_fileno = self.transport.get_extra_info('socket').fileno()
+        sock_name = self.transport.get_extra_info('sockname')
+        sock_fileno = self.transport.get_extra_info('socket').fileno()
+        self.sock_description = f"{sock_fileno} ({sock_name})"
 
     def connection_lost(self, exc: Optional[Exception]) -> None:
         """Handle connection lost."""
 
 
+def async_send_with_transport(
+    log_debug: bool,
+    transport: asyncio.DatagramTransport,
+    packet: bytes,
+    packet_num: int,
+    out: DNSOutgoing,
+    addr: Optional[str],
+    port: int,
+    v6_flow_scope: Union[Tuple[()], Tuple[int, int]] = (),
+) -> None:
+    s = transport.get_extra_info('socket')
+    ipv6_socket = s.family == socket.AF_INET6
+    if addr is None:
+        real_addr = _MDNS_ADDR6 if ipv6_socket else _MDNS_ADDR
+    else:
+        real_addr = addr
+        if not can_send_to(ipv6_socket, real_addr):
+            return
+    if log_debug:
+        log.debug(
+            'Sending to (%s, %d) via [socket %s (%s)] (%d bytes #%d) %r as %r...',
+            real_addr,
+            port or _MDNS_PORT,
+            s.fileno(),
+            transport.get_extra_info('sockname'),
+            len(packet),
+            packet_num + 1,
+            out,
+            packet,
+        )
+    # Get flowinfo and scopeid for the IPV6 socket to create a complete IPv6
+    # address tuple: https://docs.python.org/3.6/library/socket.html#socket-families
+    if ipv6_socket and not v6_flow_scope:
+        _, _, sock_flowinfo, sock_scopeid = s.getsockname()
+        v6_flow_scope = (sock_flowinfo, sock_scopeid)
+    transport.sendto(packet, (real_addr, port or _MDNS_PORT, *v6_flow_scope))
+
+
 class Zeroconf(QuietLogger):
 
     """Implementation of Zeroconf Multicast DNS Service Discovery
@@ -816,48 +854,16 @@ class Zeroconf(QuietLogger):
         # If no transport is specified, we send to all the ones
         # with the same address family
         transports = [transport] if transport else self.engine.senders
+        log_debug = log.isEnabledFor(logging.DEBUG)
 
         for packet_num, packet in enumerate(out.packets()):
             if len(packet) > _MAX_MSG_ABSOLUTE:
                 self.log_warning_once("Dropping %r over-sized packet (%d bytes) %r", out, len(packet), packet)
                 return
             for send_transport in transports:
-                self._async_send_transport(send_transport, packet, packet_num, out, addr, port, v6_flow_scope)
-
-    def _async_send_transport(
-        self,
-        transport: asyncio.DatagramTransport,
-        packet: bytes,
-        packet_num: int,
-        out: DNSOutgoing,
-        addr: Optional[str],
-        port: int,
-        v6_flow_scope: Union[Tuple[()], Tuple[int, int]] = (),
-    ) -> None:
-        s = transport.get_extra_info('socket')
-        if addr is None:
-            real_addr = _MDNS_ADDR6 if s.family == socket.AF_INET6 else _MDNS_ADDR
-        else:
-            real_addr = addr
-        if not can_send_to(s, real_addr):
-            return
-        log.debug(
-            'Sending to (%s, %d) via [socket %s (%s)] (%d bytes #%d) %r as %r...',
-            real_addr,
-            port or _MDNS_PORT,
-            s.fileno(),
-            transport.get_extra_info('sockname'),
-            len(packet),
-            packet_num + 1,
-            out,
-            packet,
-        )
-        # Get flowinfo and scopeid for the IPV6 socket to create a complete IPv6
-        # address tuple: https://docs.python.org/3.6/library/socket.html#socket-families
-        if s.family == socket.AF_INET6 and not v6_flow_scope:
-            _, _, sock_flowinfo, sock_scopeid = s.getsockname()
-            v6_flow_scope = (sock_flowinfo, sock_scopeid)
-        transport.sendto(packet, (real_addr, port or _MDNS_PORT, *v6_flow_scope))
+                async_send_with_transport(
+                    log_debug, send_transport, packet, packet_num, out, addr, port, v6_flow_scope
+                )
 
     def _close(self) -> None:
         """Set global done and remove all service listeners."""
